@@ -1,18 +1,14 @@
 #!/usr/bin/env python
-from __future__ import print_function
-from __future__ import division
-from builtins import str
-from builtins import range
-from builtins import object
 
 # standard library
+import logging
+logger = logging.getLogger(__name__)
 
 # third party
 import numpy as np
 from scipy.cluster.hierarchy import fcluster, dendrogram
 from scipy.spatial.distance import squareform
 import fastcluster
-import skbio
 
 try:
     from Bio.Cluster import kmedoids
@@ -22,12 +18,20 @@ except ImportError:
     Biopython_Unavailable = True
 
 from sklearn.cluster import AffinityPropagation, DBSCAN, KMeans
-from sklearn.mixture import GMM
-from sklearn.manifold import spectral_embedding
+import sklearn.manifold
+
+# GMM was deprecated in scikit-learn version 0.18 and fully removed in 0.20
+import pkg_resources
+sklearn_version = [int(x) for x in pkg_resources.get_distribution("scikit-learn").version.split('.')]
+USE_GAUSSIAN_MIXTURE =  tuple(sklearn_version) >= (0, 20, 0)
+if USE_GAUSSIAN_MIXTURE:
+    from sklearn.mixture import GaussianMixture
+else:
+    from sklearn.mixture import GMM
 
 # treeCl
-from .distance_matrix import DistanceMatrix, rbf, binsearch_mask, kmask, kscale, affinity, laplace, eigen, double_centre, \
-    normalise_rows, CoordinateMatrix
+from .distance_matrix import DistanceMatrix, rbf, binsearch_mask, kmask, kscale, affinity, laplace, eigen, \
+    double_centre, normalise_rows, CoordinateMatrix
 from .partition import Partition
 from .utils import enum
 from .errors import OptionError, isnumbercheck, rangecheck
@@ -113,7 +117,10 @@ class EMMixin(object):
 
     @staticmethod
     def gmm(nclusters, coords, n_init=50, n_iter=500):
-        est = GMM(n_components=nclusters, n_init=n_init, n_iter=n_iter)
+        if USE_GAUSSIAN_MIXTURE:
+            est = GaussianMixture(n_components=nclusters, n_init=n_init, max_iter=n_iter)
+        else:
+            est = GMM(n_components=nclusters, n_init=n_init, n_iter=n_iter)
         est.fit(coords)
         return Partition(est.predict(coords))
 
@@ -277,7 +284,7 @@ class Spectral(ClusteringManager, EMMixin):
         n:      int
                 The number of dimensions
         """
-        coords = spectral_embedding(self._affinity, n)
+        coords = sklearn.manifold.spectral_embedding(self._affinity, n_components=n)
         return CoordinateMatrix(normalise_rows(coords))
 
     def spectral_embedding_(self, n):
@@ -524,20 +531,10 @@ class Evaluation(ClusteringManager):
     anosim and permanova seem pretty useless; silhouette is ok
     """
     def anosim(self, partition, n_permutations=999):
-        if partition.is_minimal():
-            raise ValueError("ANOSim is not defined for singleton clusters")
-        elif partition.is_maximal():
-            raise ValueError("ANOSim is not defined for maximally divided partitions")
-        result = skbio.stats.distance.ANOSIM(skbio.DistanceMatrix(self.get_dm(False)), partition.partition_vector)
-        return result(n_permutations)
+        logger.warning("Evaluation.anosim is deprecated. Use silhouette instead.")
 
     def permanova(self, partition, n_permutations=999):
-        if partition.is_minimal():
-            raise ValueError("PERMANOVA is not defined for singleton clusters")
-        elif partition.is_maximal():
-            raise ValueError("PERMANOVA is not defined for maximally divided partitions")
-        result = skbio.stats.distance.PERMANOVA(skbio.DistanceMatrix(self.get_dm(False)), partition.partition_vector)
-        return result(n_permutations)
+        logger.warning("Evaluation.permanova is deprecated. Use silhouette instead.")
 
     def silhouette(self, partition):
         pvec   = np.array(partition.partition_vector)
