@@ -1,20 +1,10 @@
 #!/usr/bin/env python
-from __future__ import print_function
-from __future__ import absolute_import
-from __future__ import division
-from builtins import next
-from builtins import hex
-from builtins import str
-from builtins import zip
-from builtins import range
-from builtins import object
 
 # standard lib
 import glob
 import hashlib
 import itertools
 import json
-import math
 import os
 import random
 import sys
@@ -38,6 +28,7 @@ from .partition import Partition
 from .parutils import SequentialJobHandler
 from .utils import fileIO, setup_progressbar, model_translate, smooth_freqs, create_gamma_model, flatten_list
 from .utils.decorators import lazyprop
+from .utils.misc import binom_coeff
 
 # set up logging
 import logging
@@ -91,12 +82,12 @@ class RecordsHandler(object):
             trees_dir=None,
             file_format='phylip',
             header_grep=None,
-            show_progressbars=True,
+            show_progress=True,
     ):
 
         self._records = None
         self._input_files = None
-        self.show_progressbars=show_progressbars
+        self.show_progress=show_progress
 
         if records is not None:
             self.records = records
@@ -239,7 +230,7 @@ class RecordsHandler(object):
         self._input_files = files
         records = []
 
-        if self.show_progressbars:
+        if self.show_progress:
             pbar = setup_progressbar("Loading files", len(files), simple_progress=True)
             pbar.start()
 
@@ -283,9 +274,9 @@ class RecordsHandler(object):
 
             record.name = (fileIO.strip_extensions(f))
             records.append(record)
-            if self.show_progressbars:
+            if self.show_progress:
                 pbar.update(i)
-        if self.show_progressbars:
+        if self.show_progress:
             pbar.finish()
         return records
 
@@ -293,7 +284,7 @@ class RecordsHandler(object):
         """ Read a directory full of tree files, matching them up to the
         already loaded alignments """
 
-        if self.show_progressbars:
+        if self.show_progress:
             pbar = setup_progressbar("Loading trees", len(self.records))
             pbar.start()
 
@@ -312,17 +303,17 @@ class RecordsHandler(object):
                 continue
 
             finally:
-                if self.show_progressbars:
+                if self.show_progress:
                     pbar.update(i)
 
-        if self.show_progressbars:
+        if self.show_progress:
             pbar.finish()
 
     def read_parameters(self, input_dir):
         """ Read a directory full of json parameter files, matching them up to the
         already loaded alignments """
 
-        if self.show_progressbars:
+        if self.show_progress:
             pbar = setup_progressbar("Loading parameters", len(self.records))
             pbar.start()
         for i, rec in enumerate(self.records):
@@ -338,9 +329,9 @@ class RecordsHandler(object):
                 continue
 
             finally:
-                if self.show_progressbars:
+                if self.show_progress:
                     pbar.update(i)
-        if self.show_progressbars:
+        if self.show_progress:
             pbar.finish()
 
     def write_parameters(self, output_dir, gz=False):
@@ -358,7 +349,8 @@ class RecordsHandler(object):
 
 class RecordsCalculatorMixin(object):
 
-    def calc_distances(self, indices=None, task_interface=None, jobhandler=default_jobhandler, batchsize=1):
+    def calc_distances(self, indices=None, task_interface=None, jobhandler=default_jobhandler, batchsize=1,
+                       show_progress=True):
         """
         Calculate fast approximate intra-alignment pairwise distances and variances using
         ML (requires ML models to have been set up using `calc_trees`).
@@ -376,7 +368,7 @@ class RecordsCalculatorMixin(object):
         args, to_delete = task_interface.scrape_args(records)
 
         # Dispatch
-        msg = '{} estimation'.format(task_interface.name)
+        msg = '{} estimation'.format(task_interface.name) if show_progress else ''
         map_result = jobhandler(task_interface.get_task(), args, msg, batchsize)
 
         # Process results
@@ -392,7 +384,19 @@ class RecordsCalculatorMixin(object):
                 # j += 1
             # pbar.finish()
 
-    def calc_trees(self, indices=None, task_interface=None, jobhandler=default_jobhandler, batchsize=1, **kwargs):
+    def calc_trees(self, indices=None, task_interface=None, jobhandler=default_jobhandler, batchsize=1,
+                   show_progress=True, **kwargs):
+        """
+        Infer phylogenetic trees for the loaded Alignments
+
+        :param indices: Only run inference on the alignments at these given indices
+        :param task_interface: Inference tool specified via TaskInterface (default RaxmlTaskInterface)
+        :param jobhandler: Launch jobs via this JobHandler (default SequentialJobHandler; also available are
+            ThreadpoolJobHandler and ProcesspoolJobHandler for running inference in parallel)
+        :param batchsize: Batch size for Thread- or ProcesspoolJobHandlers)
+        :param kwargs: Remaining arguments to pass to the TaskInterface
+        :return: None
+        """
         if indices is None:
             indices = list(range(len(self)))
 
@@ -405,7 +409,7 @@ class RecordsCalculatorMixin(object):
         args, to_delete = task_interface.scrape_args(records, **kwargs)
 
         # Dispatch work
-        msg = '{} Tree estimation'.format(task_interface.name)
+        msg = '{} Tree estimation'.format(task_interface.name) if show_progress else ''
         map_result = jobhandler(task_interface.get_task(), args, msg, batchsize)
 
         # Process results
@@ -414,8 +418,23 @@ class RecordsCalculatorMixin(object):
                 #logger.debug('Result - {}'.format(result))
                 rec.parameters.construct_from_dict(result)
 
-    def get_inter_tree_distances(self, metric, jobhandler=default_jobhandler, normalise=False, min_overlap=4, batchsize=1):
-        """ Generate a distance matrix from a fully-populated Collection """
+    def get_inter_tree_distances(self, metric, jobhandler=default_jobhandler,
+                                 normalise=False, min_overlap=4, overlap_fail_value=0,
+                                 batchsize=1, show_progress=True):
+        """ Generate a distance matrix from a fully-populated Collection.
+            Can silence progressbars with show_progress=False option
+        :param metric: str. Tree distance metric to use. Choice of 'euc', 'geo', 'rf', 'wrf'.
+        :param jobhandler: treeCl.Jobhandler. Choice of SequentialJobHandler, ThreadpoolJobHandler, or
+            ProcesspoolJobHandler.
+        :param normalise:  Bool. Whether to normalise the tree distance to the size of the leaf set.
+        :param min_overlap: int. Trees with fewer leaves in common than this threshold will not have their distance
+            calculated, but instead the distance returned will be the value in `overlap_fail_value`.
+        :param overlap_fail_value: Any. The distance between trees with fewer leaves in common than `min_overlap`
+            is set to this value.
+        :param batchsize: int. Number of jobs to process in a batch when using a ProcesspoolJobHandler or a
+            ThreadpoolJobHandler.
+        :return: treeCl.DistanceMatrix.
+        """
         metrics = {'euc': tasks.EuclideanTreeDistance,
                    'geo': tasks.GeodesicTreeDistance,
                    'rf': tasks.RobinsonFouldsTreeDistance,
@@ -430,10 +449,10 @@ class RecordsCalculatorMixin(object):
             trees = (PhyloTree(newick, False) for newick in self.trees)
         else:
             trees = self.trees
-        args = task_interface.scrape_args(trees, normalise, min_overlap)
+        args = task_interface.scrape_args(trees, normalise, min_overlap, overlap_fail_value)
         logger.debug('{}'.format(args))
-        msg = task_interface.name
-        array = jobhandler(task_interface.get_task(), args, msg, batchsize)
+        msg = task_interface.name if show_progress else ''
+        array = jobhandler(task_interface.get_task(), args, msg, batchsize, nargs=binom_coeff(len(trees)))
         return DistanceMatrix.from_array(squareform(array), self.names)
 
 
