@@ -1,10 +1,4 @@
 #!/usr/bin/env python
-from __future__ import print_function
-from __future__ import division
-from builtins import str
-from builtins import range
-from builtins import object
-from past.utils import old_div
 
 # standard lib
 import sys
@@ -13,7 +7,6 @@ import sys
 import numpy as np
 import pandas as pd
 import sklearn
-from scipy.spatial.distance import squareform
 
 # treeCl
 from . import errors
@@ -333,10 +326,29 @@ def _embedding_spectral(matrix, dimensions=3, unit_length=True,
         aff = rbf(matrix, sigma=sigma)
     else:
         aff = affinity_matrix
-    coords = sklearn.manifold.spectral_embedding(aff, dimensions)
+    coords = sklearn.manifold.spectral_embedding(aff, n_components=dimensions)
     return normalise_rows(coords) if unit_length else coords
 
 
+def _embedding_tsne(matrix, dimensions=3, early_exaggeration=12.0,
+                    method='barnes_hut', perplexity=30, learning_rate=200,
+                    n_iter=1000):
+    """
+    Private method to perform tSNE embedding
+    :param matrix: treeCl Distance Matrix
+    :param dimensions: Number of dimensions in which to embed points
+    :return: treeCl CoordinateMatrix
+    """
+    tsne = sklearn.manifold.TSNE(n_components=dimensions,
+                                 metric="precomputed",
+                                 early_exaggeration=early_exaggeration,
+                                 method=method,
+                                 perplexity=perplexity,
+                                 learning_rate=learning_rate,
+                                 n_iter=1000)
+    return tsne.fit_transform(matrix)
+
+
 def _embedding_metric_mds(matrix, dimensions=3):
     """
     Private method to calculate MMDS embedding
@@ -450,7 +462,9 @@ class DistanceMatrix(Matrix):
     def from_csv(cls, filename, **kwargs):
         with fileIO.freader(filename) as handle:
             new_instance = cls()
-            new_instance.df = pd.DataFrame.from_csv(handle, **kwargs)
+            if not 'index_col' in kwargs:
+                kwargs['index_col'] = 0  # new behaviour in pandas
+            new_instance.df = pd.read_csv(handle, **kwargs)
         return new_instance
 
     def to_csv(self, filename, **kwargs):
@@ -507,6 +521,7 @@ class DistanceMatrix(Matrix):
             mmds: Metric MultiDimensional Scaling
             nmmds: Non-Metric MultiDimensional Scaling
             spectral: Spectral decomposition of Laplacian matrix
+            tsne: t-distributed Stochastic Neighbour Embedding
 
         Valid kwargs:
             kpca: affinity_matrix - a precomputed array of affinities
@@ -522,7 +537,7 @@ class DistanceMatrix(Matrix):
         :param kwargs: unit_length (bool), affinity_matrix (np.array), sigma (float), initial_coords (np.array)
         :return: coordinate matrix (np.array)
         """
-        errors.optioncheck(method, ['cmds', 'kpca', 'mmds', 'nmmds', 'spectral'])
+        errors.optioncheck(method, ['cmds', 'kpca', 'mmds', 'nmmds', 'spectral', 'tsne'])
         if method == 'cmds':
             array =  _embedding_classical_mds(self.to_array(), dimensions, **kwargs)
         elif method == 'kpca':
@@ -533,6 +548,8 @@ class DistanceMatrix(Matrix):
             array = _embedding_nonmetric_mds(self.to_array(), dimensions, **kwargs)
         elif method == 'spectral':
             array = _embedding_spectral(self.to_array(), dimensions, **kwargs)
+        elif method == 'tsne':
+            array = _embedding_tsne(self.to_array(), dimensions, **kwargs)
 
         return CoordinateMatrix(array, names=self.df.index)
 
