@@ -1,8 +1,3 @@
-from __future__ import absolute_import
-from builtins import zip
-from builtins import str
-from builtins import range
-from builtins import object
 import itertools
 import json
 import tempfile
@@ -23,12 +18,12 @@ from .constants import RANDOM_SEED
 from .wrappers.phylogenetics import FastTree, parse_fasttree_output, Raxml, Phyml
 from .parsers import RaxmlParser, PhymlParser
 import logging
-from future.utils import with_metaclass
+
 logger = logging.getLogger(__name__)
 import numpy as np
 
 
-class TaskInterface(with_metaclass(ABCMeta, object)):
+class TaskInterface(metaclass=ABCMeta):
     _name = None
 
     @abstractmethod
@@ -43,41 +38,41 @@ class TaskInterface(with_metaclass(ABCMeta, object)):
     def name(self):
         return self._name
 
-def eucdist_task(newick_string_a, newick_string_b, normalise, min_overlap=4):
+def eucdist_task(newick_string_a, newick_string_b, normalise, min_overlap=4, overlap_fail_value=0):
     """
     Distributed version of tree_distance.eucdist
     Parameters: two valid newick strings and a boolean
     """
     tree_a = Tree(newick_string_a)
     tree_b = Tree(newick_string_b)
-    return treedist.eucdist(tree_a, tree_b, normalise, min_overlap)
+    return treedist.eucdist(tree_a, tree_b, normalise, min_overlap, overlap_fail_value)
 
-def geodist_task(newick_string_a, newick_string_b, normalise, min_overlap=4):
+def geodist_task(newick_string_a, newick_string_b, normalise, min_overlap=4, overlap_fail_value=0):
     """
     Distributed version of tree_distance.geodist
     Parameters: two valid newick strings and a boolean
     """
     tree_a = Tree(newick_string_a)
     tree_b = Tree(newick_string_b)
-    return treedist.geodist(tree_a, tree_b, normalise, min_overlap)
+    return treedist.geodist(tree_a, tree_b, normalise, min_overlap, overlap_fail_value)
 
-def rfdist_task(newick_string_a, newick_string_b, normalise, min_overlap=4):
+def rfdist_task(newick_string_a, newick_string_b, normalise, min_overlap=4, overlap_fail_value=0):
     """
     Distributed version of tree_distance.rfdist
     Parameters: two valid newick strings and a boolean
     """
     tree_a = Tree(newick_string_a)
     tree_b = Tree(newick_string_b)
-    return treedist.rfdist(tree_a, tree_b, normalise, min_overlap)
+    return treedist.rfdist(tree_a, tree_b, normalise, min_overlap, overlap_fail_value)
 
-def wrfdist_task(newick_string_a, newick_string_b, normalise, min_overlap=4):
+def wrfdist_task(newick_string_a, newick_string_b, normalise, min_overlap=4, overlap_fail_value=0):
     """
     Distributed version of tree_distance.rfdist
     Parameters: two valid newick strings and a boolean
     """
     tree_a = Tree(newick_string_a)
     tree_b = Tree(newick_string_b)
-    return treedist.wrfdist(tree_a, tree_b, normalise, min_overlap)
+    return treedist.wrfdist(tree_a, tree_b, normalise, min_overlap, overlap_fail_value)
 
 ### TASKS that calculate trees
 def pll_task(alignment_file, partition_string, guidetree=None, tree_search=True, threads=1, seed=RANDOM_SEED, frequencies=None,
@@ -192,22 +187,22 @@ def fasttree_task(alignment_file, dna=False):
     result['partitions'][0]['model'] = 'GTR' if dna else 'WAG'
     return result
 
-def raxml_task(executable, alignment_file, model, partitions_file=None, outfile=None, threads=1, parsimony=False, fast_tree=False):
+def raxml_task(executable, alignment_file, model, partitions_file=None, outfile=None, threads=1, parsimony=False, fast_tree=False, n_starts=1):
     logger.debug('raxml_task: executable {}, alignment_file {}, model {}, partitions_file {}, outfile {}, threads {}, parsimony {}, fast_tree {}'.format(executable, alignment_file, model, partitions_file, outfile, threads, parsimony, fast_tree))
     afl = os.path.abspath(alignment_file)
     pfl = os.path.abspath(partitions_file) if partitions_file else None
     if threads > 1:
         if 'raxmlHPC' in executable and not 'PTHREADS' in executable:
             executable = executable.replace('raxmlHPC', 'raxmlHPC-PTHREADS')
+            logger.warn("Sequential executable modified because threading requested: {}".format(executable))
         basecmd = '-T {} '.format(threads)
-        logger.debug("Sequential executable modified because threading requested: {}".format(executable))
     else:
         basecmd = ''
 
     # initialise RAxML wrapper
     rax = Raxml(executable, verbose=False)
 
-    with fileIO.TempDir() as tmpd, fileIO.TempFile(tmpd) as name:
+    with fileIO.TempDir(disable_delete=False) as tmpd, fileIO.TempFile(tmpd, disable_delete=False) as name:
         name = os.path.basename(name)
         seed=random.randint(1000, 9999)
         outdir=os.path.abspath(tmpd)
@@ -217,6 +212,16 @@ def raxml_task(executable, alignment_file, model, partitions_file=None, outfile=
         cmd = basecmd + '-m {model} -n {name} -s {seqfile} -p {seed} -O -w {outdir}'.format(
             model=model, name=name, seqfile=afl, seed=seed,
             outdir=outdir)
+
+        try:
+            n_starts = int(n_starts)
+        except ValueError:
+            logger.warn('Value given for n_starts ("{}") is not an integer. Setting n_starts=1'.format(n_starts))
+            n_starts = 1
+
+        if n_starts > 1:
+            cmd += ' -N {}'.format(int(n_starts))
+
         if pfl:
             cmd += ' -q {}'.format(pfl)
         if fast_tree:
@@ -248,7 +253,8 @@ def raxml_task(executable, alignment_file, model, partitions_file=None, outfile=
         elif parsimony:
             # Need to follow up
             parsimony_tree_file = os.path.join(outdir, 'RAxML_parsimonyTree.{}'.format(name))
-            logger.debug('Parsimony tree file exists {}'.format('yes' if os.path.exists(parsimony_tree_file) else 'no'))
+            logger.debug('Parsimony tree file exists {}'
+                         .format('yes' if os.path.exists(parsimony_tree_file) else 'no'))
             cmd = basecmd + '-m {model} -n modopt -s {seqfile} -p {seed} -O -w {outdir} -f e -t {parsimony}'.format(
                 model=model, seqfile=afl, seed=seed, outdir=outdir, parsimony=parsimony_tree_file)
             if pfl:
@@ -263,7 +269,7 @@ def raxml_task(executable, alignment_file, model, partitions_file=None, outfile=
 
         else:
             info_file = os.path.join(outdir, 'RAxML_info.{}'.format(name))
-            result_file = os.path.join(outdir, 'RAxML_result.{}'.format(name))
+            result_file = os.path.join(outdir, 'RAxML_bestTree.{}'.format(name))
             dash_f_e = False
 
         logger.debug('Info file found - {}'.format('yes' if os.path.exists(info_file) else 'no'))
@@ -272,11 +278,13 @@ def raxml_task(executable, alignment_file, model, partitions_file=None, outfile=
 
         if not os.path.exists(info_file):
             info_file = os.path.join(os.path.abspath('.'), 'RAxML_info.{}'.format(name))
-            logger.debug('Fallback info file ({}) found - {}'.format(info_file, 'yes' if os.path.exists(info_file) else 'no'))
+            logger.debug('Fallback info file ({}) found - {}'
+                         .format(info_file, 'yes' if os.path.exists(info_file) else 'no'))
 
         if not os.path.exists(result_file):
             result_file = os.path.join(os.path.abspath('.'), 'RAxML_result.{}'.format(name))
-            logger.debug('Fallback result file ({}) found - {}'.format(result_file, 'yes' if os.path.exists(result_file) else 'no'))
+            logger.debug('Fallback result file ({}) found - {}'
+                         .format(result_file, 'yes' if os.path.exists(result_file) else 'no'))
 
         parser = RaxmlParser()
         result = parser.to_dict(info_file, result_file, dash_f_e=dash_f_e)
@@ -423,9 +431,34 @@ class BionjTaskInterface(TaskInterface):
 
 
 class RaxmlTaskInterface(TaskInterface):
+    """
+    Provides a high-level interface to the RAxML tree inference program.
+    Using the interface is a two-stage process. First, call the `scrape_args`
+    method on a list of `Alignment` objects to generate a list of command line
+    arguments for RAxML, and a list of temporary files that will need cleaning
+    up. Second, use the `get_task` method to access a `raxml_task` function that
+    can be passed, with the argument list, to a JobHandler to run the jobs.
+    """
     _name = 'Raxml'
 
-    def scrape_args(self, records, executable='raxmlHPC-AVX', partition_files=None, model=None, outfiles=None, threads=1, parsimony=False, fast_tree=False):
+    def scrape_args(self, records, executable='raxmlHPC-AVX', partition_files=None,
+                    model=None, outfiles=None, threads=1, parsimony=False, fast_tree=False,
+                    n_starts=1):
+        """
+        Examine a list of records and generate RAxML command line arguments for tree inference.
+        :param records: list of `Alignment` records
+        :param executable: name of the RAxML executable on the system to use. Must be in the user's path.
+        :param partition_files: List of RAxML partition files used to describe any partitioning scheme
+            to be used (optional)
+        :param model: Choice of model to use. Defaults to GTRGAMMA for DNA, or PROTGAMMALGX for amino acid alignments.
+        :param outfiles: A list of output file locations to write results (required length = 1 per alignment)
+        :param threads: Number of threads for RAxML to use. This is independent of any threading used by the
+            `JobHandler`, and the user should be sure that their choice is appropriate for the number of threads
+            available to their system, and for the RAxML executable being used.
+        :param parsimony: Use RAxML's parsimony tree search only
+        :param fast_tree: Use RAxML's experimental fast tree search (-f E)
+        :return: (List of command line arguments, List of created temporary files)
+        """
         args = []
         to_delete = []
         if partition_files is None:
@@ -458,10 +491,14 @@ class RaxmlTaskInterface(TaskInterface):
                             name=rec.name, seqlen=len(rec))
                         tmpfile.write(partition_string)
 
-            args.append((executable, filename, model, qfile, ofile, threads, parsimony, fast_tree))
+            args.append((executable, filename, model, qfile, ofile, threads, parsimony, fast_tree, n_starts))
         return args, to_delete
 
     def get_task(self):
+        """
+        Access the RAxML command line interface wrapper
+        :return: `raxml_task` function used to dispatch jobs to system RAxML
+        """
         return raxml_task
 
 
@@ -608,11 +645,13 @@ class SimulatorTaskInterface(TaskInterface):
                                result['ml_tree'],
                                partition['rates'] if 'rates' in partition else None)
 
-class TreeDistanceTaskInterface(with_metaclass(ABCMeta, TaskInterface)):
+class TreeDistanceTaskInterface(metaclass=ABCMeta):
     _name = 'TreeDistance'
 
-    def scrape_args(self, trees, normalise=False, min_overlap=4):
-        return [(t1, t2, normalise, min_overlap) for (t1, t2) in itertools.combinations(trees, 2)]
+    def scrape_args(self, trees, normalise, min_overlap, overlap_fail_value):
+        for (t1, t2) in itertools.combinations(trees, 2):
+            yield (t1, t2, normalise, min_overlap, overlap_fail_value)
+
 
     @abstractmethod
     def get_task(self):
