@@ -1,6 +1,4 @@
 #!/usr/bin/env python
-from __future__ import print_function
-from builtins import zip
 from ..alignment import Alignment
 
 docstring = '''
