@@ -1,5 +1,3 @@
-from builtins import zip
-from builtins import range
 def enum(*sequential, **named):
     """creates an Enum type with given values"""
     enums = dict(list(zip(sequential, list(range(len(sequential))))), **named)
