@@ -1,8 +1,3 @@
-from __future__ import division, print_function
-from builtins import zip
-from builtins import str
-from builtins import object
-
 import numpy as np
 
 from ..partition import Partition
