@@ -1,7 +1,3 @@
-from __future__ import print_function
-from __future__ import division
-from builtins import object
-from past.utils import old_div
 import numpy as np
 import pandas as pd
 from ..partition import Partition
